<?php
require_once 'include.php';

ini_set( 'precision', 14 );
ini_set( 'serialize_precision', 14 );

//use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

if (!defined('_PS_VERSION_'))
    exit;

class CredibanCo extends PaymentModule
{
    const _REGISTERED_BUT_NOT_PAID_FOR_ = 0;
    const _PRE_AUTHORIZATION_ = 1;
    const _AUTHORIZATION_ = 2;
    const _AUTHORIZATION_CANCELED_ = 3;
    const _ORDER_RETURN_ = 4;
    const _ACS_INITIATED_AUTHORIZATION_THROUGH_THE_ISSUING_BANK_ = 5;
    const _AUTHORIZATION_FAILED_ = 6;
    static $cache = array();
    public $default_order_state;
    public $test_mode;
    public $send_order = 0;
    public $ext = '.do';
    public $write_log;
//    public $discountHelper;

    protected $login;
    protected $password;
    protected $real_link = RBS_CREDIBANCO_PROD_URL;
    protected $debug_link = RBS_CREDIBANCO_TEST_URL;


    public function __construct()
    {
        $this->name = 'credibanco';
        $this->tab = 'payments_gateways';
        $this->version = '1.0.0';
        $this->author = 'CredibanCo';
        $this->controllers = array('payment', 'validation');
        $this->currencies = true;
        $this->currencies_mode = 'checkbox';
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = 'CredibanCo';
        $this->description = $this->l('Allows you to use a payment gateway CredibanCo', 'credibanco');
        $this->confirmUninstall = $this->l('Are you sure to delete this module', 'credibanco');

        $this->default_order_state = Configuration::get('RBS_CREDIBANCO_STATE_DEFAULT');
        $this->login = Configuration::get('RBS_CREDIBANCO_LOGIN');
        $this->password = Configuration::get('RBS_CREDIBANCO_PASSWORD');
        $this->test_mode = Configuration::get('RBS_CREDIBANCO_TEST_MODE');
        $this->send_order = Configuration::get('RBS_CREDIBANCO_SEND_ORDER');
        $this->default_vat = Configuration::get('RBS_CREDIBANCO_DEFAULT_VAT');
        $this->stage = Configuration::get('RBS_CREDIBANCO_STAGE');
        $this->write_log = Configuration::get('RBS_CREDIBANCO_WRITE_LOG');
        $this->default_vat = 0;
    }

    /**
     * @param string $pattern
     * @param int $flags
     * @return array
     */
    public static function globRecursive($pattern, $flags = 0)
    {
        $files = glob($pattern, $flags);
        if (!$files)
            $files = array();

        foreach (glob(dirname($pattern) . '/*', GLOB_ONLYDIR | GLOB_NOSORT) as $dir)
            /** @noinspection SlowArrayOperationsInLoopInspection */
            $files = array_merge($files, self::globRecursive($dir . '/' . basename($pattern), $flags));

        return $files;
    }

    public static function noEscape($value)
    {
        return $value;
    }

    public function install()
    {
        $sql = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'credibanco_payment` (
                  `id_order` int(11) NOT NULL,
                  `order_number` text COLLATE utf8_unicode_ci NOT NULL,
                  `rbs_order_id` text COLLATE utf8_unicode_ci NOT NULL,
                  `form_url` text COLLATE utf8_unicode_ci NOT NULL,
                  PRIMARY KEY (`id_order`),
                  UNIQUE KEY `id_order` (`id_order`)
                ) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;';

        if (!parent::install()
            || !$this->registerHook('payment')
            || !$this->registerHook('paymentReturn')
            || !$this->registerHook('paymentOptions')
            || !Db::getInstance()->execute($sql)
        ) {
            return false;
        }

        return true;
    }

    public function uninstall()
    {
        $sql = 'DROP TABLE IF EXISTS `' . _DB_PREFIX_ . 'credibanco_payment`';
        if (!parent::uninstall()
            || !Db::getInstance()->execute($sql)
        )
            return false;
        return true;
    }

    public function hookDisplayPaymentReturn()
    {
        $this->hookPayment();
    }

    public function hookPayment()
    {
        if (!$this->active)
            return;

        $this->smarty->assign(array(
            'module' => $this,
            'this_path' => $this->_path,
            'this_path_ssl' => Tools::getShopDomainSsl(true, true) . __PS_BASE_URI__ . 'modules/' . $this->name . '/'
        ));

        $this->context->controller->addCSS($this->_path . 'views/css/credibanco.css', 'all');
        return $this->display(__FILE__, 'payment.tpl');
    }

    public function hookPaymentOptions($params)
    {
        if (!$this->active) {
            return;
        }

        if (!$this->checkCurrency($params['cart'])) {
            return;
        }

        $this->smarty->assign(array(
            'module' => $this,
            'this_path' => $this->_path,
            'this_path_ssl' => Tools::getShopDomainSsl(true, true) . __PS_BASE_URI__ . 'modules/' . $this->name . '/'
        ));

        $newOption = new PaymentOption();
        $title = $this->l('Online payment with CredibanCo', 'credibanco');
        $newOption->setCallToActionText($title)
            ->setAction($this->context->link->getModuleLink($this->name, 'validation', array(), true))
            ->setAdditionalInformation($this->fetch('module:credibanco/views/templates/hook/payment.tpl'));
        $payment_options = array(
            $newOption
        );

        return $payment_options;
    }

    public function checkCurrency($cart)
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);

        if (is_array($currencies_module))
            foreach ($currencies_module as $currency_module)
                if ($currency_order->id == $currency_module['id_currency'])
                    return true;
        return false;
    }

    public function checkCartAmount($cart = null)
    {
        if (empty($cart)) {
            return false;
        }

        if (!(int)Configuration::get('RBS_CREDIBANCO_SEND_ORDER')) {
            return true;
        }

        $discounts = $cart->getOrderTotal(true, Cart::ONLY_DISCOUNTS);
        $shipping = $cart->getOrderTotal(true, Cart::ONLY_SHIPPING);

        if (($discounts == $shipping) || ($discounts == 0)) {
            return true;
        }

        return false;
    }

    public function registerInPaymentGate($id_order)
    {
        $cache_key = 'getPaymentGateValues_' . $id_order;

        if (isset(self::$cache[$cache_key]))
            unset(self::$cache[$cache_key]);

        $order = new Order((int)$id_order);
        $order_number = Db::getInstance()->getValue('SELECT IFNULL(`order_number`, 0) FROM `'
                . _DB_PREFIX_ . 'credibanco_payment` WHERE `id_order` = \'' . (int)$order->id . '\'') + 1;

        $rbs_description = "Orden de pago " . $id_order . " \n " . $order->getFirstMessage();

        $curl_data = array(
            'userName' => $this->getLogin(),
            'password' => $this->getPassword(),
            'amount' => $order->total_paid * 100,
            'orderNumber' => (int)$order->id . '_' . $order_number . '.' . time(),
            'description' => $rbs_description,
            'returnUrl' => $this->context->link->getModuleLink($this->name, 'paymentsuccess', array(), true),
            'failUrl' => $this->context->link->getModuleLink($this->name, 'paymentfail', array(), true),
        );

        $jsonParams_array = array(
            'CMS' => 'PrestaShop ' . _PS_VERSION_,
            'Module-Version' => 'CredibanCo ' . $this->version
        );

        // Customer data
        $id_customer = $order->id_customer;
        $customer = new Customer((int)$id_customer);

        if (!empty($customer->email)) {
            $jsonParams_array['email'] = $customer->email;
        }

        $user_address = new Address((int)$order->id_address_invoice);

        $user_phone = (!empty($user_address->phone) ? $user_address->phone : $user_address->phone_mobile);
        if (!empty($user_phone)) {
            $jsonParams_array['phone'] = "+" . preg_replace("/(\W*)/", "", $user_phone);
        }

        $jsonParams_array['payerCity'] = $user_address->city;
        $jsonParams_array['postAddress'] = $user_address->address1;

        $state = new State((int) $user_address->id_state);
        if (!empty($state->name)){
            $jsonParams_array['payerState'] = $state->name;
        }

        $jsonParams_array['payerPostalCode'] = $user_address->postcode;

        $delivery_address = new Address((int)$order->id_address_delivery);
        if (!empty($delivery_address)) {
            $delivery_phone = (!empty($delivery_address->phone) ? $delivery_address->phone : $delivery_address->phone_mobile);
            $jsonParams_array['shippingAddress'] = $delivery_address->address1;
//            $jsonParams_array['shippingContactEmail'] = '';
            $jsonParams_array['shippingContactPhone'] = "+" . preg_replace("/(\W*)/", "", $delivery_phone);
        }

        //collect orderBundle data
        {
            $OrderDetailObject = new OrderDetail;
            $product_detail = $OrderDetailObject->getList($id_order);


            $items = array();
            $itemsCnt = 1;

            $amountFix = 0;

            $iva_amount_total = 0;
            $iac_amount_total = 0;

            foreach ($product_detail as $value) {
                $item = array();

                $item['positionId'] = $itemsCnt++;
                $item['name'] = $value['product_name'];
                $item['quantity'] = array(
                    'value' => $value['product_quantity'],
                    'measure' => RBS_CREDIBANCO_MEASUREMENT_NAME
                );
                $item['itemAmount'] = round(($value['unit_price_tax_incl']) * 100) * $value['product_quantity'];
                $item['itemCode'] = $value['product_id'] . "_" . $itemsCnt;
                $item['itemPrice'] = round(($value['unit_price_tax_incl']) * 100);

                // TAX calculation
                if ($value['id_tax_rules_group'] > 0) {
                    $tg_obj = new TaxRulesGroup($value['id_tax_rules_group']);
                    if (Validate::isLoadedObject($tg_obj)) {
                        $address = $this->context->shop->getAddress();
                        $tax_manager = TaxManagerFactory::getManager($address, $value['id_tax_rules_group']);
                        $product_tax_calculator = $tax_manager->getTaxCalculator();
                        $item_rate = $product_tax_calculator->getTotalRate();
                    }

                    if (strtolower($tg_obj->name) == "iva") {
                        $iva_amount_total += ($value['unit_price_tax_incl'] - $value['unit_price_tax_excl']) * $value['product_quantity'] * 100;
                        $jsonParams_array["IVA.amount"] = $iva_amount_total;
                    }

                    if (strtolower($tg_obj->name) == "iac") {
                        $iac_amount_total += ($value['unit_price_tax_incl'] - $value['unit_price_tax_excl']) * $value['product_quantity'] * 100;
                        $jsonParams_array["IAC.amount"] = $iac_amount_total;
                    }
                }
                $items[] = $item;
            }

            // DELIVERY calculation new position add
            if ($order->total_shipping > 0) {
                $itemShipment['positionId'] = $itemsCnt;
                $itemShipment['name'] = 'Delivery';
                $itemShipment['quantity'] = array(
                    'value' => 1,
                    'measure' => RBS_CREDIBANCO_MEASUREMENT_NAME
                );
                $itemShipment['itemAmount'] = $itemShipment['itemPrice'] = $order->total_shipping * 100;
                $itemShipment['itemCode'] = 'Delivery';

                $attributes = array();
                $attributes[] = array(
                    "name" => "paymentMethod",
                    "value" => 4
                );
                $attributes[] = array(
                    "name" => "paymentObject",
                    "value" => 4
                );

                $itemShipment['itemAttributes']['attributes'] = $attributes;


                $items[] = $itemShipment;
            }

            $order_bundle = array(
                'orderCreationDate' => strtotime($order->date_add),
                'customerDetails' => array(
                    'email' => $customer->email
                ),
                'cartItems' => array('items' => $items)
            );

            $curl_data['amount'] = $order->total_paid * 100;;

        }
        $curl_data['jsonParams'] = json_encode($jsonParams_array);

        if ($this->getStage() == 1) {
            $result_gate = $this->gateway('registerPreAuth' . $this->ext, $curl_data, $this->getLink(), $this->write_log);
        } else {
            $result_gate = $this->gateway('register' . $this->ext, $curl_data, $this->getLink(), $this->write_log);
        }

        $result = $this->noCamelCase($result_gate);

        if (!isset($result['errorcode']) || !$result['errorcode']) {

            $sql = 'INSERT INTO `' . _DB_PREFIX_ . 'credibanco_payment` (`id_order`, `order_number`, `rbs_order_id`, `form_url`)
                        VALUES (\'' . (int)$order->id . '\', \'' . pSQL($order_number) . '\' , \'' . pSQL($result['orderid']) . '\', \'' . pSQL($result['formurl']) . '\')
                        ON DUPLICATE KEY UPDATE
                            `order_number` = \'' . pSQL($order_number) . '\',
                            `rbs_order_id` = \'' . (int)$result['orderid'] . '\',
                            `form_url` = \'' . pSQL($result['formurl']) . '\'';
            return Db::getInstance()->execute($sql);

        }
        return $result;
    }

    public function getLogin()
    {
        return $this->login;
    }

    public function getPassword()
    {
        return $this->password;
    }

    public function getStage()
    {
        return $this->stage;
    }

    public function callback_addresses_update($action_adr)
    {
        // $action_adr is prod/test URL
        $callback_addresses_string = $this->context->link->getModuleLink($this->name, 'paymentcallback', array(), true);

        $gate_url = str_replace("payment/rest", "mportal/mvc/public/merchant/update", $action_adr);
        $gate_url .= substr($this->login, 0, -4); // we guess username = login w/o "-api"

        $headers = array(
            'Content-Type:application/json',
            'Authorization: Basic ' . base64_encode($this->login . ":" . $this->password)
        );
        $data['callbacks_enabled'] = true;
        $data['callback_addresses'] = $callback_addresses_string;
        $data['callback_operations'] = "deposited,approved,declinedByTimeout";

        $ch = curl_init();
        curl_setopt_array($ch, array(
//                CURLOPT_HTTPAUTH => CURLAUTH_BASIC,
            CURLOPT_HTTPHEADER => $headers,
            CURLOPT_VERBOSE => true,
            CURLOPT_SSL_VERIFYHOST => false,
            CURLOPT_SSL_VERIFYPEER => false,
            CURLOPT_URL => $gate_url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_POST => true,
            CURLOPT_POSTFIELDS => json_encode($data),
            CURLOPT_ENCODING, "gzip",
            CURLOPT_ENCODING, '',
        ));
        $response = curl_exec($ch);
        $this->logger("[callback_addresses_string]: " . $callback_addresses_string ."\n" . print_r($data, true) . 'Response: ' . print_r($response, true));
    }

    public function gateway($method, $data, $url, $log = true)
    {

        if($method == "register.do") {
            $this->callback_addresses_update($url);
        }

        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_SSL_VERIFYHOST => false,
            CURLOPT_SSL_VERIFYPEER => false,
            CURLOPT_URL => $url . $method,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_POST => true,
            CURLOPT_POSTFIELDS => http_build_query($data, '', '&'),
            CURLOPT_HTTPHEADER => array('CMS: PrestaShop ' . _PS_VERSION_, 'Module-Version: ' . $this->version),
            CURLOPT_SSLVERSION => 6
        ));
        $response = curl_exec($curl);
        $response = json_decode($response, true);

//        if ($log == true)
            $this->logger('Request: ' . $url . $method . ': ' . print_r($data, true) . 'Response: ' . print_r($response, true));

        curl_close($curl);

        return $response;
    }

    public function logger($message)
    {
        $logger = new FileLogger();
        $dir = version_compare(_PS_VERSION_, '1.7', '<') ? '/log/' : '/app/logs/';
        $logger->setFilename(_PS_ROOT_DIR_ . $dir . date('Y-m') . '_credibanco.log');
        $logger->logInfo(date('H:i:s') . "\n" . $message);
    }

    public function getLink()
    {
        return ((int)$this->test_mode) ? $this->debug_link : $this->real_link;
    }

    public function noCamelCase($obj)
    {
        $return = array();
        if (is_object($obj) || is_array($obj)) {
            foreach ($obj as $key => $value) {
                $k = Tools::strtolower($key);
                $return[$k] = $value;
            }
            return $return;
        }
        return false;
    }

    public function getAdditionalActions($params)
    {
        if ($params['id_order_state'] != Configuration::get('RBS_CREDIBANCO_STATE'))
            if ($params['id_order_state'] != Configuration::get('RBS_CREDIBANCO_STATE_DEFAULT'))
                return;

        $info = $this->getPaymentGateValues($params['id_order']);

        $this->context->smarty->assign(array(
            'payment_link' => $this->context->link->getModuleLink($this->name, 'PaymentGate'),
            'id_order' => $params['id_order'],
            'order_status' => $info['order_status']
        ));
        return $this->display(__FILE__, 'views/templates/front/additional_action.tpl');
    }

    public function getPaymentGateValues($id_order)
    {
        $cache_key = 'getPaymentGateValues_' . $id_order;

        if (!isset(self::$cache[$cache_key])) {

            $result = Db::getInstance()->getRow('SELECT * FROM `' . _DB_PREFIX_ . 'credibanco_payment` WHERE `id_order` = \'' . (int)$id_order . '\'');

            if ($result) {
                $curl_data = array('userName' => $this->getLogin(), 'password' => $this->getPassword(), 'orderId' => $result['rbs_order_id']);
                $return_gate = $this->gateway('getOrderStatus' . $this->ext, $curl_data, $this->getLink(), $this->write_log);

                $return = $this->noCamelCase($return_gate);

                if (isset($return['orderstatus'])) {
                    $result['order_status'] = $return['orderstatus'];
                    self::$cache[$cache_key] = $result;
                }
            } else
                self::$cache[$cache_key] = false;
        }
        return self::$cache[$cache_key];
    }

    public function getRedirectLink()
    {
        return $this->context->link->getModuleLink($this->name, 'PaymentGate', array(
            'id_order' => $this->currentOrder
        ));
    }

    public function getContent()
    {
        $this->registerSmartyFunctions();
        $this->context->controller->addJS('http://#'
            . Context::getContext()->language->iso_code
        );
        $this->context->smarty->assign(array(
            'content_tab' => $this->getContentWrap(),
//			'documentation' => $this->getDocumentation()
        ));

        return $this->context->smarty->fetch(_PS_MODULE_DIR_ . $this->name . '/views/templates/admin/content.tpl');
    }

    public function registerSmartyFunctions()
    {
        $smarty = $this->context->smarty;
        if (!array_key_exists('get_image_lang', $smarty->registered_plugins['function']))
            smartyRegisterFunction($smarty, 'function', 'get_image_lang', array($this, 'getImageLang'));
        if (!array_key_exists('no_escape', $smarty->registered_plugins['modifier']))
            smartyRegisterFunction($smarty, 'modifier', 'no_escape', array(__CLASS__, 'noEscape'));
        if (class_exists('TransModRbs')) {
            if (!array_key_exists('ld', $smarty->registered_plugins['modifier']))
                smartyRegisterFunction($smarty, 'modifier', 'ld', array(TransModRbs::getInstance(), 'ld'));
        }
    }

    /**
     * @return string
     */
    public function getContentWrap()
    {
        $this->context->controller->addCSS($this->_path . 'views/css/credibanco.css', 'all');
        if (Tools::isSubmit('saveSettings')) {
            $this->login = Tools::getValue('api-login');
            $this->password = Tools::getValue('password');
            $this->stage = Tools::getValue('stage');
            $this->test_mode = Tools::getValue('test_mode');
            $this->send_order = Tools::getValue('send_order');
            $this->write_log = Tools::getValue('write_log');
            $this->default_vat = Tools::getValue('default_vat');

            Configuration::updateValue('RBS_CREDIBANCO_LOGIN', $this->login);
            Configuration::updateValue('RBS_CREDIBANCO_PASSWORD', $this->password);
            Configuration::updateValue('RBS_CREDIBANCO_STAGE', $this->stage);
            Configuration::updateValue('RBS_CREDIBANCO_TEST_MODE', $this->test_mode);
            Configuration::updateValue('RBS_CREDIBANCO_STATE', Tools::getValue('authorization_state'));
            Configuration::updateValue('RBS_CREDIBANCO_STATE_DEFAULT', Tools::getValue('default_state'));
            Configuration::updateValue('RBS_CREDIBANCO_STATE_ERROR', Tools::getValue('error_state'));

            Configuration::updateValue('RBS_CREDIBANCO_SEND_ORDER', $this->send_order);
            Configuration::updateValue('RBS_CREDIBANCO_DEFAULT_TAX_SYSTEM', Tools::getValue('default_taxSystem'));
            Configuration::updateValue('RBS_CREDIBANCO_DEFAULT_VAT', $this->default_vat);

//            Configuration::updateValue('RBS_CREDIBANCO_DEFAULT_FFD_VERSION', Tools::getValue('ffd_version'));
//            Configuration::updateValue('RBS_CREDIBANCO_DEFAULT_FFD_PAYMENT_OBJECT_TYPE', Tools::getValue('ffd_paymentObjectType'));
//            Configuration::updateValue('RBS_CREDIBANCO_DEFAULT_FFD_PAYMENT_METHOD_TYPE', Tools::getValue('ffd_paymentMethodType'));

            Configuration::updateValue('RBS_CREDIBANCO_WRITE_LOG', $this->write_log);
        }

        $query = 'SELECT `id_order_state`, `name` FROM `' . _DB_PREFIX_ . 'order_state_lang` WHERE `id_lang` = "' . (int)$this->context->language->id . '"';
        $order_state = DB::getInstance()->executeS($query);

        return $this->adminForm($order_state);
    }

    public function adminForm($order_state)
    {
        $fields_form = array(
            array(
                'label' => 'API login',
//                'desc' => '',
                'required' => true,
                'type' => 'text',
                'name' => 'api-login',
                'class' => 'fixed-width-lg',
                'tab' => 'basic',
            ),
            array(
                'label' => $this->l('Password', 'credibanco'), //'',
//                'desc' => '',
                'required' => true,
                'type' => 'text',
                'name' => 'password',
                'class' => 'fixed-width-lg',
                'tab' => 'basic',
            ),
            array(
                'label' => $this->l('Test mode', 'credibanco'), //'Modo de prueba',
//                'desc' => '',
                'type' => 'radio',
                'name' => 'test_mode',
                'values' => array(
                    array(
                        'value' => 1,
                        'label' => $this->l('Enabled'),
                        'p' => '&nbsp;'
                    ),
                    array(
                        'value' => 0,
                        'label' => $this->l('Disabled'),
                        'p' => '&nbsp;'
                    )
                ),
                'tab' => 'basic',
            ),
            array(
                'label' => $this->l('Payments mode', 'credibanco'), //'Tipo de pago',
//                'desc' => '',
                'name' => 'stage',

                'type' => 'select',
                'options' => array(
                    'id' => 'id',
                    'name' => 'name',
                    'query' => array(
                        array('id' => 1, 'name' => $this->l('Two-phase payments', 'credibanco')), //'Pagos en dos pasos'),
                        array('id' => 0, 'name' => $this->l('One-phase payments', 'credibanco')), //'Pagos en un solo paso'),
                    ),
                ),
                'tab' => 'basic',
            ),


            array(
                'label' => $this->l('Default state', 'credibanco'), // 'Estado predeterminado',
                'desc' => $this->l('Default order status for this payment method', 'credibanco'), // 'Estado de pedido predeterminado para este método de pago',
                'type' => 'select',
                'name' => 'default_state',
                'options' => array(
                    'id' => 'id_order_state',
                    'name' => 'name',
                    'query' => $order_state
                ),
                'tab' => 'basic',
            ),
            array(
                'label' => $this->l('Success payment state', 'credibanco'), //'Estado de pago del pedido',
//                'desc' => '',
                'type' => 'select',
                'name' => 'authorization_state',
                'options' => array(
                    'id' => 'id_order_state',
                    'name' => 'name',
                    'query' => $order_state
                ),
                'tab' => 'basic',
            ),
            array(
                'label' => $this->l('Error payment state', 'credibanco'), //'Estado para error',
                'desc' => $this->l('Order status in case of payment error', 'credibanco'), //'Estado del pedido en caso de error de pago',
                'type' => 'select',
                'name' => 'error_state',
                'options' => array(
                    'id' => 'id_order_state',
                    'name' => 'name',
                    'query' => $order_state
                ),
                'tab' => 'basic',
            ),

            array(
                'label' => $this->l('Keep a log of requests', 'credibanco'), //'Registro de solicitudes y respuestas de la pasarela',
                'type' => 'radio',
                'name' => 'write_log',
                'values' => array(
                    array(
                        'value' => 1,
                        'label' => $this->l('Enabled'),
                        'p' => '&nbsp;'
                    ),
                    array(
                        'value' => 0,
                        'label' => $this->l('Disabled'),
                        'p' => '&nbsp;'
                    )
                ),
                'tab' => 'basic',
            ),
        );


        $fields_value = array(
            'api-login' => Configuration::get('RBS_CREDIBANCO_LOGIN') ? Configuration::get('RBS_CREDIBANCO_LOGIN') : null,
            'password' => Configuration::get('RBS_CREDIBANCO_PASSWORD') ? Configuration::get('RBS_CREDIBANCO_PASSWORD') : null,
            'test_mode' => Configuration::get('RBS_CREDIBANCO_TEST_MODE') ? Configuration::get('RBS_CREDIBANCO_TEST_MODE') : '0',
            'stage' => Configuration::get('RBS_CREDIBANCO_STAGE') ? Configuration::get('RBS_CREDIBANCO_STAGE') : '0',

            'default_state' => Configuration::get('RBS_CREDIBANCO_STATE_DEFAULT') ? Configuration::get('RBS_CREDIBANCO_STATE_DEFAULT') : null,
            'authorization_state' => Configuration::get('RBS_CREDIBANCO_STATE') ? Configuration::get('RBS_CREDIBANCO_STATE') : null,
            'error_state' => Configuration::get('RBS_CREDIBANCO_STATE_ERROR') ? Configuration::get('RBS_CREDIBANCO_STATE_ERROR') : null,

            'send_order' => Configuration::get('RBS_CREDIBANCO_SEND_ORDER') ? Configuration::get('RBS_CREDIBANCO_SEND_ORDER') : '0',
//            'default_taxSystem' => Configuration::get('RBS_CREDIBANCO_DEFAULT_TAX_SYSTEM') ? Configuration::get('RBS_CREDIBANCO_DEFAULT_TAX_SYSTEM') : null,
//            'default_vat' => Configuration::get('RBS_CREDIBANCO_DEFAULT_VAT') ? Configuration::get('RBS_CREDIBANCO_DEFAULT_VAT') : null,
//            'ffd_version' => Configuration::get('RBS_CREDIBANCO_DEFAULT_FFD_VERSION') ? Configuration::get('RBS_CREDIBANCO_DEFAULT_FFD_VERSION') : null,

//            'ffd_paymentObjectType' => Configuration::get('RBS_CREDIBANCO_DEFAULT_FFD_PAYMENT_OBJECT_TYPE') ? Configuration::get('RBS_CREDIBANCO_DEFAULT_FFD_PAYMENT_OBJECT_TYPE') : null,
//            'ffd_paymentMethodType' => Configuration::get('RBS_CREDIBANCO_DEFAULT_FFD_PAYMENT_METHOD_TYPE') ? Configuration::get('RBS_CREDIBANCO_DEFAULT_FFD_PAYMENT_METHOD_TYPE') : null,

            'write_log' => Configuration::get('RBS_CREDIBANCO_WRITE_LOG') ? Configuration::get('RBS_CREDIBANCO_WRITE_LOG') : '0',
        );

        $fields[0]['form'] = array(
            'legend' => array(
                'title' => $this->l('Allow customers to conveniently checkout directly with  ', 'credibanco') // 'Configurar la aceptación de pagos electrónicos '
            ),
            'input' => $fields_form,
            'submit' => array(
                'title' => $this->l('Save'),
                'name' => 'saveSettings',
                'class' => 'button btn btn-default pull-right',
                'desc' => ''
            )
        );

        $helper_form = new HelperForm();
        $helper_form->fields_value = $fields_value;
        $helper_form->token = Tools::getValue('token');
        $helper_form->currentIndex = 'index.php?controller=AdminModules&configure=' . $this->name
            . '&tab_module=front_office_features&module_name=' . $this->name;

        return $helper_form->generateForm($fields);
    }

    public function getImageLang($smarty)
    {
        if (_PS_VERSION_ < 1.5)
            $cookie = &$GLOBALS['cookie'];
        else {
            $cookie = $this->context->cookie;
            $cookie->id_lang = $this->context->language->id;
        }

        $path = $smarty['path'];
        $module_path = $this->name . '/views/img/';
        $current_language = new Language($cookie->id_lang);
        $module_lang_path = $module_path . $current_language->iso_code . '/';
        $module_lang_default_path = $module_path . 'en/';
        $path_image = false;

        if (file_exists(_PS_MODULE_DIR_ . $module_lang_path . $path))
            $path_image = _MODULE_DIR_ . $module_lang_path . $path;

        elseif (file_exists(_PS_MODULE_DIR_ . $module_lang_default_path . $path))
            $path_image = _MODULE_DIR_ . $module_lang_default_path . $path;

        if ($path_image)
            return '<img class="thumbnail" src="' . $path_image . '">';
        else
            return '[can not load image "' . $path . '"]';
    }
}